@@ -63,7 +63,6 @@ extern "C" {
 
 /* Type definitions for the basic structures.
  */
-typedef struct _GdkKeyInfo            GdkKeyInfo;
 typedef struct _GdkPoint	      GdkPoint;
 typedef struct _GdkRectangle	      GdkRectangle;
 typedef struct _GdkSegment	      GdkSegment;
@@ -105,6 +104,8 @@ typedef struct _GdkDrawable           GdkDrawable;
 typedef struct _GdkDrawable           GdkBitmap;
 typedef struct _GdkDrawable           GdkPixmap;
 typedef struct _GdkDrawable           GdkWindow;
+typedef struct _GdkDisplay	      GdkDisplay;
+typedef struct _GdkScreen	      GdkScreen;
 
 typedef enum
 {
@@ -131,7 +132,7 @@ typedef enum
   GDK_BUTTON5_MASK  = 1 << 12,
   /* The next few modifiers are used by XKB, so we skip to the end
    */
-  GDK_RELEASE_MASK  = 1u << 31,
+  GDK_RELEASE_MASK  = 1 << 30,
   GDK_MODIFIER_MASK = GDK_RELEASE_MASK | 0x1fff
 } GdkModifierType;
 
@@ -170,16 +171,6 @@ typedef void (*GdkInputFunction) (gpointer	    data,
 
 typedef void (*GdkDestroyNotify) (gpointer data);
 
-/* GdkKeyInfo is a description of a the hardware key and state that
- * can be mapped to some keysym.
- */
-struct _GdkKeyInfo
-{
-  guint keycode;
-  gint  group;
-  gint  level;
-};
-
 struct _GdkPoint
 {
   gint x;
