@@ -378,12 +378,13 @@ gdk_drag_begin (GdkWindow     *window,
 }
 
 guint32
-gdk_drag_get_protocol (guint32          xid,
-		       GdkDragProtocol *protocol)
+gdk_drag_get_protocol_for_display (GdkDisplay      *display,
+				   guint32          xid,
+				   GdkDragProtocol *protocol)
 {
   GdkWindow *window;
 
-  window = gdk_window_lookup (xid);
+  window = gdk_window_lookup ((GdkNativeWindow) xid);
 
   if (GPOINTER_TO_INT (gdk_drawable_get_data (window, "gdk-dnd-registered")))
     {
@@ -392,7 +393,7 @@ gdk_drag_get_protocol (guint32          xid,
     }
   
   *protocol = GDK_DRAG_PROTO_NONE;
-  return GDK_NONE;
+  return 0;
 }
 
 static GdkWindow *
@@ -444,12 +445,13 @@ get_toplevel_window_at (GdkWindow *ignore,
 
 
 void
-gdk_drag_find_window (GdkDragContext  *context,
-		      GdkWindow       *drag_window,
-		      gint             x_root,
-		      gint             y_root,
-		      GdkWindow      **dest_window,
-		      GdkDragProtocol *protocol)
+gdk_drag_find_window_for_screen (GdkDragContext  *context,
+				 GdkWindow       *drag_window,
+				 GdkScreen       *screen,
+				 gint             x_root,
+				 gint             y_root,
+				 GdkWindow      **dest_window,
+				 GdkDragProtocol *protocol)
 {
   GdkWindow *dest;
 
@@ -466,7 +468,7 @@ gdk_drag_find_window (GdkDragContext  *context,
       /* Check if new destination accepts drags, and which protocol */
       if ((recipient = gdk_drag_get_protocol ((guint32)dest, protocol)))
 	{
-	  *dest_window = gdk_window_lookup (recipient);
+	  *dest_window = gdk_window_lookup ((GdkNativeWindow) recipient);
 	  gdk_window_ref (*dest_window);
 	}
       else
