@@ -70,6 +70,7 @@ struct _GdkKeymapX11
   guint lock_keysym;
   GdkModifierType group_switch_mask;
   GdkModifierType num_lock_mask;
+  gboolean sun_keypad;
   PangoDirection current_direction;
   gboolean have_direction;
   guint current_serial;
@@ -124,6 +125,7 @@ gdk_keymap_x11_init (GdkKeymapX11 *keymap)
   keymap->mod_keymap = NULL;
   
   keymap->num_lock_mask = 0;
+  keymap->sun_keypad = FALSE;
   keymap->group_switch_mask = 0;
   keymap->lock_keysym = GDK_Caps_Lock;
   keymap->have_direction = FALSE;
@@ -185,6 +187,8 @@ get_xkb (GdkKeymapX11 *keymap_x11)
  * @returns: the #GdkKeymap attached to @display.
  *
  * Returns the #GdkKeymap attached to @display.
+ *
+ * Since: 2.2
  **/
 GdkKeymap*
 gdk_keymap_get_for_display (GdkDisplay *display)
@@ -283,69 +287,86 @@ update_keymaps (GdkKeymapX11 *keymap_x11)
       keymap_x11->group_switch_mask = 0;
       keymap_x11->num_lock_mask = 0;
 
-      /* there are 8 modifiers, and the first 3 are shift, shift lock,
-       * and control
+      /* There are 8 sets of modifiers, with each set containing
+       * max_keypermod keycodes.
        */
       map_size = 8 * keymap_x11->mod_keymap->max_keypermod;
-      i = 3 * keymap_x11->mod_keymap->max_keypermod;
-      while (i < map_size)
+      for (i = 0; i < map_size; i++)
         {
-          /* get the key code at this point in the map,
-           * see if its keysym is GDK_Mode_switch, if so
-           * we have the mode key
-           */
+          /* Get the key code at this point in the map. */
           gint keycode = keymap_x11->mod_keymap->modifiermap[i];
-      
-          if (keycode >= keymap_x11->min_keycode &&
-              keycode <= keymap_x11->max_keycode)
-            {
-              gint j;
-              KeySym *syms = keymap_x11->keymap + (keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
-	      /* GDK_MOD1_MASK is 1 << 3 for example, i.e. the
-	       * fourth modifier, i / keyspermod is the modifier
-	       * index
-	       */
-	      guint mask = 1 << ( i / keymap_x11->mod_keymap->max_keypermod);
+          gint j;
+          KeySym *syms;
+	  guint mask;
 
-	      if (mask == GDK_LOCK_MASK)
+          /* Ignore invalid keycodes. */
+          if (keycode < keymap_x11->min_keycode ||
+              keycode > keymap_x11->max_keycode)
+            continue;
+
+          syms = keymap_x11->keymap + (keycode - keymap_x11->min_keycode) * keymap_x11->keysyms_per_keycode;
+
+          /* The fourth modifier, GDK_MOD1_MASK is 1 << 3.
+	   * Each group of max_keypermod entries refers to the same modifier.
+           */
+          mask = 1 << (i / keymap_x11->mod_keymap->max_keypermod);
+
+          switch (mask)
+            {
+            case GDK_LOCK_MASK:
+              /* Get the Lock keysym.  If any keysym bound to the Lock modifier
+               * is Caps_Lock, we will interpret the modifier as Caps_Lock;
+               * otherwise, if any is bound to Shift_Lock, we will interpret
+               * the modifier as Shift_Lock. Otherwise, the lock modifier
+	       * has no effect.
+               */
+	      for (j = 0; j < keymap_x11->keysyms_per_keycode; j++)
 		{
-		  for (j = 0; j < keymap_x11->keysyms_per_keycode; j++)
-		    {
-		      if (syms[j] == GDK_Caps_Lock)
-			keymap_x11->lock_keysym = syms[j];
-		      else if (syms[j] == GDK_Shift_Lock && keymap_x11->lock_keysym == GDK_VoidSymbol)
-			keymap_x11->lock_keysym = syms[j];
-		    }
+		  if (syms[j] == GDK_Caps_Lock)
+		    keymap_x11->lock_keysym = GDK_Caps_Lock;
+		  else if (syms[j] == GDK_Shift_Lock &&
+			   keymap_x11->lock_keysym == GDK_VoidSymbol)
+		    keymap_x11->lock_keysym = GDK_Shift_Lock;
 		}
-	      
-	      /* Some keyboard maps are known to map Mode_Switch as an extra
-	       * Mod1 key. In circumstances like that, it won't be used to
-	       * switch groups.
-	       */
-	      if (mask == GDK_CONTROL_MASK || mask == GDK_SHIFT_MASK ||
-		  mask == GDK_LOCK_MASK || mask == GDK_MOD1_MASK)
-		goto next;
-	      
+              break;
+
+            case GDK_CONTROL_MASK:
+            case GDK_SHIFT_MASK:
+            case GDK_MOD1_MASK:
+              /* Some keyboard maps are known to map Mode_Switch as an
+               * extra Mod1 key. In circumstances like that, it won't be
+               * used to switch groups.
+               */
+              break;
+
+            default:
+              /* Find the Mode_Switch and Num_Lock modifiers. */
               for (j = 0; j < keymap_x11->keysyms_per_keycode; j++)
                 {
                   if (syms[j] == GDK_Mode_switch)
                     {
                       /* This modifier swaps groups */
-
                       keymap_x11->group_switch_mask |= mask;
                     }
-                  if (syms[j] == GDK_Num_Lock)
+                  else if (syms[j] == GDK_Num_Lock)
                     {
-                      /* This modifier swaps groups */
-
+                      /* This modifier is used for Num_Lock */
                       keymap_x11->num_lock_mask |= mask;
                     }
                 }
+              break;
             }
-
-	next:
-          ++i;
         }
+
+      /* Hack: The Sun X server puts the keysym to use when the Num Lock
+       * modifier is on in the third element of the keysym array, instead
+       * of the second.
+       */
+      if ((strcmp (ServerVendor (xdisplay), "Sun Microsystems, Inc.") == 0) &&
+          (keymap_x11->keysyms_per_keycode > 2))
+        keymap_x11->sun_keypad = TRUE;
+      else
+        keymap_x11->sun_keypad = FALSE;
     }
 }
 
@@ -839,9 +860,12 @@ gdk_keymap_lookup_key (GdkKeymap          *keymap,
 }
 
 #ifdef HAVE_XKB
-/* This is copied straight from XFree86 Xlib, because I needed to
- * add the group and level return. It's unchanged for ease of
- * diff against the Xlib sources; don't reformat it.
+/* This is copied straight from XFree86 Xlib, to:
+ *  - add the group and level return.
+ *  - change the interpretation of mods_rtrn as described
+ *    in the docs for gdk_keymap_translate_keyboard_state()
+ * It's unchanged for ease of diff against the Xlib sources; don't
+ * reformat it.
  */
 static Bool
 MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
@@ -895,29 +919,48 @@ MyEnhancedXkbTranslateKeyCode(register XkbDescPtr     xkb,
     if (type->map) { /* find the column (shift level) within the group */
         register int i;
         register XkbKTMapEntryPtr entry;
+	/* ---- Begin section modified for GDK  ---- */
+	int found = 0;
+	
         for (i=0,entry=type->map;i<type->map_count;i++,entry++) {
-            if ((entry->active)&&((mods&type->mods.mask)==entry->mods.mask)) {
+	    if (mods_rtrn) {
+		int bits = 0;
+		unsigned long tmp = entry->mods.mask;
+		while (tmp) {
+		    if ((tmp & 1) == 1)
+			bits++;
+		    tmp >>= 1;
+		}
+		/* We always add one-modifiers levels to mods_rtrn since
+		 * they can't wipe out bits in the state unless the
+		 * level would be triggered. But return other modifiers
+		 * 
+		 */
+		if (bits == 1 || (mods&type->mods.mask)==entry->mods.mask)
+		    *mods_rtrn |= entry->mods.mask;
+	    }
+	    
+            if (!found&&entry->active&&((mods&type->mods.mask)==entry->mods.mask)) {
                 col+= entry->level;
                 if (type->preserve)
                     preserve= type->preserve[i].mask;
 
-                /* ---- Begin stuff GDK adds to the original Xlib version ---- */
-                
                 if (level_rtrn)
                   *level_rtrn = entry->level;
                 
-                /* ---- End stuff GDK adds to the original Xlib version ---- */
-                
-                break;
+                found = 1;
             }
         }
+	/* ---- End section modified for GDK ---- */
     }
 
     if (keysym_rtrn!=NULL)
         *keysym_rtrn= syms[col];
     if (mods_rtrn) {
-        *mods_rtrn= type->mods.mask&(~preserve);
-
+	/* ---- Begin section modified for GDK  ---- */
+        *mods_rtrn &= ~preserve;
+	/* ---- End section modified for GDK ---- */
+	
         /* ---- Begin stuff GDK comments out of the original Xlib version ---- */
         /* This is commented out because xkb_info is a private struct */
 
@@ -967,6 +1010,7 @@ translate_keysym (GdkKeymapX11   *keymap_x11,
   GdkModifierType shift_modifiers;
   gint shift_level;
   guint tmp_keyval;
+  gint num_lock_index;
 
   shift_modifiers = GDK_SHIFT_MASK;
   if (keymap_x11->lock_keysym == GDK_Shift_Lock)
@@ -978,13 +1022,34 @@ translate_keysym (GdkKeymapX11   *keymap_x11,
       (SYM (keymap_x11, 0, 0) || SYM (keymap_x11, 0, 1)))
     group = 0;
 
-  if ((state & GDK_SHIFT_MASK) != 0 &&
-      KEYSYM_IS_KEYPAD (SYM (keymap_x11, group, 1)))
+  /* Hack: On Sun, the Num Lock modifier uses the third element in the
+   * keysym array, and Mode_Switch does not apply for a keypad key.
+   */
+  if (keymap_x11->sun_keypad)
+    {
+      num_lock_index = 2;
+      
+      if (group != 0)
+	{
+	  gint i;
+	  
+	  for (i = 0; i < keymap_x11->keysyms_per_keycode; i++)
+	    if (KEYSYM_IS_KEYPAD (SYM (keymap_x11, 0, i)))
+	      group = 0;
+	}
+    }
+  else
+    num_lock_index = 1;
+
+  if ((state & keymap_x11->num_lock_mask) &&
+      KEYSYM_IS_KEYPAD (SYM (keymap_x11, group, num_lock_index)))
     {
       /* Shift, Shift_Lock cancel Num_Lock
        */
-       shift_level = (state & shift_modifiers) ? 0 : 1;
-      
+      shift_level = (state & shift_modifiers) ? 0 : num_lock_index;
+      if (!SYM (keymap_x11, group, shift_level) && SYM (keymap_x11, group, 0))
+        shift_level = 0;
+
        tmp_keyval = SYM (keymap_x11, group, shift_level);
     }
   else
@@ -998,7 +1063,7 @@ translate_keysym (GdkKeymapX11   *keymap_x11,
   
       tmp_keyval = SYM (keymap_x11, group, shift_level);
       
-      if (keymap_x11->lock_keysym == GDK_Caps_Lock && (state & GDK_SHIFT_MASK) != 0)
+      if (keymap_x11->lock_keysym == GDK_Caps_Lock && (state & GDK_LOCK_MASK) != 0)
 	{
 	  guint upper = gdk_keyval_to_upper (tmp_keyval);
 	  if (upper != tmp_keyval)
@@ -1038,6 +1103,49 @@ translate_keysym (GdkKeymapX11   *keymap_x11,
  * affected by the active keyboard group. The @level is derived from
  * @state. For convenience, #GdkEventKey already contains the translated
  * keyval, so this function isn't as useful as you might think.
+ *
+ * <note><para>
+ * @consumed_modifiers gives modifiers that should be masked out
+ * from @state when comparing this key press to a hot key. For
+ * instance, on a US keyboard, the <literal>plus</literal>
+ * symbol is shifted, so when comparing a key press to a
+ * <literal>&lt;Control&gt;plus</literal> accelerator &lt;Shift&gt; should
+ * be masked out.
+ * </para>
+ * <informalexample><programlisting>
+ * &sol;* We want to ignore irrelevant modifiers like ScrollLock *&sol;
+ * #define ALL_ACCELS_MASK (GDK_CONTROL_MASK | GDK_SHIFT_MASK | GDK_MOD1_MASK)
+ * gdk_keymap_translate_keyboard_state (keymap, event->hardware_keycode,
+ *                                      event->state, event->group,
+ *                                      &amp;keyval, NULL, NULL, &amp;consumed);
+ * if (keyval == GDK_PLUS &amp;&amp;
+ *     (event->state &amp; ~consumed &amp; ALL_ACCELS_MASK) == GDK_CONTROL_MASK)
+ *   &sol;* Control was pressed *&sol;
+ * </programlisting></informalexample>
+ * <para>
+ * An older interpretation @consumed_modifiers was that it contained
+ * all modifiers that might affect the translation of the key;
+ * this allowed accelerators to be stored with irrelevant consumed
+ * modifiers, by doing:</para>
+ * <informalexample><programlisting>
+ * &sol;* XXX Don't do this XXX *&sol;
+ * if (keyval == accel_keyval &amp;&amp;
+ *     (event->state &amp; ~consumed &amp; ALL_ACCELS_MASK) == (accel_mods &amp; ~consumed))
+ *   &sol;* Accelerator was pressed *&sol;
+ * </programlisting></informalexample>
+ * <para>
+ * However, this did not work if multi-modifier combinations were
+ * used in the keymap, since, for instance, <literal>&lt;Control&gt;</literal>
+ * would be masked out even if only <literal>&lt;Control&gt;&lt;Alt&gt;</literal>
+ * was used in the keymap. To support this usage as well as well as
+ * possible, all <emphasis>single modifier</emphasis> combinations
+ * that could affect the key for any combination of modifiers will
+ * be returned in @consumed_modifiers; multi-modifier combinations
+ * are returned only when actually found in @state. When you store
+ * accelerators, you should always store them with consumed modifiers
+ * removed. Store <literal>&lt;Control&gt;plus</literal>,
+ * not <literal>&lt;Control&gt;&lt;Shift&gt;plus</literal>,
+ * </para></note>
  * 
  * Return value: %TRUE if there was a keyval bound to the keycode/state/group
  **/
