@@ -38,8 +38,8 @@
 
 #define CHILD_SPACING     1
 
-static GtkBorder default_default_border = { 1, 1, 1, 1 };
-static GtkBorder default_default_outside_border = { 0, 0, 0, 0 };
+static const GtkBorder default_default_border = { 1, 1, 1, 1 };
+static const GtkBorder default_default_outside_border = { 0, 0, 0, 0 };
 
 /* Time out before giving up on getting a key release when animatng
  * the close button.
