@@ -357,7 +357,7 @@ gtk_tree_view_column_finalize (GObject *object)
   g_list_free (tree_column->cell_list);
 
   if (tree_column->child)
-    g_object_unref (G_OBJECT (tree_column->child));
+    g_object_unref (tree_column->child);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -978,8 +978,8 @@ _gtk_tree_view_column_realize_button (GtkTreeViewColumn *column)
   attr.wclass = GDK_INPUT_ONLY;
   attr.visual = gtk_widget_get_visual (GTK_WIDGET (tree_view));
   attr.colormap = gtk_widget_get_colormap (GTK_WIDGET (tree_view));
-  attr.event_mask = gtk_widget_get_events (GTK_WIDGET (tree_view));
-  attr.event_mask = (GDK_BUTTON_PRESS_MASK |
+  attr.event_mask = gtk_widget_get_events (GTK_WIDGET (tree_view)) |
+                    (GDK_BUTTON_PRESS_MASK |
 		     GDK_BUTTON_RELEASE_MASK |
 		     GDK_POINTER_MOTION_MASK |
 		     GDK_POINTER_MOTION_HINT_MASK |
@@ -1103,8 +1103,9 @@ _gtk_tree_view_column_count_special_cells (GtkTreeViewColumn *column)
     {
       GtkTreeViewColumnCellInfo *cellinfo = list->data;
 
-      if (cellinfo->cell->mode == GTK_CELL_RENDERER_MODE_EDITABLE ||
-	  cellinfo->cell->mode == GTK_CELL_RENDERER_MODE_ACTIVATABLE)
+      if ((cellinfo->cell->mode == GTK_CELL_RENDERER_MODE_EDITABLE ||
+	  cellinfo->cell->mode == GTK_CELL_RENDERER_MODE_ACTIVATABLE) &&
+	  cellinfo->cell->visible)
 	i++;
     }
 
@@ -1622,9 +1623,9 @@ gtk_tree_view_column_set_resizable (GtkTreeViewColumn *tree_column,
  * gtk_tree_view_column_get_resizable:
  * @tree_column: A #GtkTreeViewColumn
  * 
- * Returns #TRUE if the @tree_column can be resized by the end user.
+ * Returns %TRUE if the @tree_column can be resized by the end user.
  * 
- * Return value: #TRUE, if the @tree_column can be resized.
+ * Return value: %TRUE, if the @tree_column can be resized.
  **/
 gboolean
 gtk_tree_view_column_get_resizable (GtkTreeViewColumn *tree_column)
@@ -2109,7 +2110,7 @@ gtk_tree_view_column_set_sort_column_id (GtkTreeViewColumn *tree_column,
 					 gint               sort_column_id)
 {
   g_return_if_fail (GTK_IS_TREE_VIEW_COLUMN (tree_column));
-  g_return_if_fail (sort_column_id >= 0);
+  g_return_if_fail (sort_column_id >= -1);
 
   if (tree_column->sort_column_id == sort_column_id)
     return;
@@ -2119,6 +2120,8 @@ gtk_tree_view_column_set_sort_column_id (GtkTreeViewColumn *tree_column,
   /* Handle unsetting the id */
   if (sort_column_id == -1)
     {
+      GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_column->tree_view));
+
       if (tree_column->sort_clicked_signal)
 	{
 	  g_signal_handler_disconnect (tree_column, tree_column->sort_clicked_signal);
@@ -2127,12 +2130,13 @@ gtk_tree_view_column_set_sort_column_id (GtkTreeViewColumn *tree_column,
 
       if (tree_column->sort_column_changed_signal)
 	{
-	  g_signal_handler_disconnect (tree_column, tree_column->sort_column_changed_signal);
+	  g_signal_handler_disconnect (model, tree_column->sort_column_changed_signal);
 	  tree_column->sort_column_changed_signal = 0;
 	}
 
       gtk_tree_view_column_set_sort_order (tree_column, GTK_SORT_ASCENDING);
       gtk_tree_view_column_set_sort_indicator (tree_column, FALSE);
+      gtk_tree_view_column_set_clickable (tree_column, FALSE);
       return;
     }
 
@@ -2357,7 +2361,7 @@ gtk_tree_view_column_cell_get_size (GtkTreeViewColumn *tree_column,
       if (visible == FALSE)
 	continue;
 
-      if (first_cell == FALSE && *width)
+      if (first_cell == FALSE && width)
 	*width += tree_column->spacing;
 
       gtk_cell_renderer_get_size (info->cell,
@@ -2408,8 +2412,9 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
   gint extra_space;
   gint min_x, min_y, max_x, max_y;
   gint focus_line_width;
-  gint dx;
   gint special_cells;
+  gint horizontal_separator;
+  gboolean cursor_row = FALSE;
 
   min_x = G_MAXINT;
   min_y = G_MAXINT;
@@ -2442,13 +2447,15 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 	}
     }
 
+  cursor_row = flags & GTK_CELL_RENDERER_FOCUSED;
+
   gtk_widget_style_get (GTK_WIDGET (tree_column->tree_view),
 			"focus-line-width", &focus_line_width,
+                        "horizontal-separator", &horizontal_separator,
 			NULL);
 
   real_cell_area = *cell_area;
   real_background_area = *background_area;
-  dx = real_cell_area.x - real_background_area.x - focus_line_width;
 
   real_cell_area.x += focus_line_width;
 
@@ -2465,7 +2472,7 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
       full_requested_width += info->requested_width;
     }
 
-  extra_space = background_area->width - full_requested_width;
+  extra_space = cell_area->width + horizontal_separator - full_requested_width;
   if (extra_space < 0)
     extra_space = 0;
   else if (extra_space > 0 && expand_cell_count > 0)
@@ -2482,14 +2489,16 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
       if (! info->cell->visible)
 	continue;
 
-      if (info->has_focus)
+      if ((info->has_focus || special_cells == 1) && cursor_row)
 	flags |= GTK_CELL_RENDERER_FOCUSED;
+      else
+        flags &= ~GTK_CELL_RENDERER_FOCUSED;
 
-      real_background_area.width = info->requested_width +
-	(info->expand?extra_space:0);
-      info->real_width = real_background_area.width;
+      info->real_width = info->requested_width + (info->expand?extra_space:0);
 
-      real_cell_area.width = real_background_area.width;
+      real_cell_area.width = info->real_width;
+      real_background_area.width=
+        real_cell_area.x + real_cell_area.width - real_background_area.x;
       real_cell_area.width -= 2 * focus_line_width;
 
       if (action == CELL_ACTION_RENDER)
@@ -2563,7 +2572,7 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 	    {
 	      gboolean visible, mode;
 
-	      g_object_get (G_OBJECT (info->cell),
+	      g_object_get (info->cell,
 			    "visible", &visible,
 			    "mode", &mode,
 			    NULL);
@@ -2608,8 +2617,8 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 
       flags &= ~GTK_CELL_RENDERER_FOCUSED;
 
-      real_cell_area.x += (info->real_width + tree_column->spacing);
-      real_background_area.x += (info->real_width + tree_column->spacing);
+      real_cell_area.x += (real_cell_area.width + tree_column->spacing);
+      real_background_area.x += (real_background_area.width + tree_column->spacing);
     }
 
   /* iterate list for PACK_END cells */
@@ -2623,14 +2632,16 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
       if (! info->cell->visible)
 	continue;
 
-      if (info->has_focus)
+      if ((info->has_focus || special_cells == 1) && cursor_row)
 	flags |= GTK_CELL_RENDERER_FOCUSED;
+      else
+        flags &= ~GTK_CELL_RENDERER_FOCUSED;
 
-      real_background_area.width = info->requested_width +
-	(info->expand?extra_space:0);
-      info->real_width = real_background_area.width;
+      info->real_width = info->requested_width + (info->expand?extra_space:0);
 
-      real_cell_area.width = real_background_area.width;
+      real_cell_area.width = info->real_width;
+      real_background_area.width =
+        real_cell_area.x + real_cell_area.width - real_background_area.x;
       real_cell_area.width -= 2 * focus_line_width;
 
       if (action == CELL_ACTION_RENDER)
@@ -2704,7 +2715,7 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 	    {
 	      gboolean visible, mode;
 
-	      g_object_get (G_OBJECT (info->cell),
+	      g_object_get (info->cell,
 			    "visible", &visible,
 			    "mode", &mode,
 			    NULL);
@@ -2748,8 +2759,8 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 
       flags &= ~GTK_CELL_RENDERER_FOCUSED;
 
-      real_cell_area.x += (info->real_width + tree_column->spacing);
-      real_background_area.x += (info->real_width + tree_column->spacing);
+      real_cell_area.x += (real_cell_area.width + tree_column->spacing);
+      real_background_area.x += (real_background_area.width + tree_column->spacing);
     }
 
   /* fill focus_rectangle when required */
@@ -3253,7 +3264,8 @@ _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *column,
 	  if (info->cell == cell)
 	    break;
 
-	  *left += info->real_width;
+	  if (info->cell->visible)
+	    *left += info->real_width;
 	}
     }
 
@@ -3283,12 +3295,27 @@ _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *column,
 	      GtkTreeViewColumnCellInfo *info =
 	        (GtkTreeViewColumnCellInfo *)list->data;
 
-	      *right += info->real_width;
+	      if (info->cell->visible)
+	        *right += info->real_width;
 	    }
 	}
     }
 }
 
+/**
+ * gtk_tree_view_column_cell_get_position:
+ * @tree_column: a #GtkTreeViewColumn
+ * @cell_renderer: a #GtkCellRenderer
+ * @start_pos: return location for the horizontal position of @cell within
+ *            @tree_column, may be %NULL
+ * @width: return location for the width of @cell, may be %NULL
+ *
+ * Obtains the horizontal position and size of a cell in a column. If the
+ * cell is not found in the column, @start_pos and @width are not changed and
+ * %FALSE is returned.
+ * 
+ * Return value: %TRUE if @cell belongs to @tree_column.
+ */
 gboolean
 gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
 					GtkCellRenderer   *cell_renderer,
@@ -3309,7 +3336,9 @@ gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
           found_cell = TRUE;
           break;
         }
-      current_x += cellinfo->real_width;
+
+      if (cellinfo->cell->visible)
+        current_x += cellinfo->real_width;
     }
 
   if (found_cell)
@@ -3322,3 +3351,5 @@ gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
 
   return found_cell;
 }
+
+
